import re
import glob
import os
import numpy as np
import copy
from pytopdrawer.topplot import Join, TopPlot
import re
from smpl import plot
import pandas as pd
import matplotlib.pyplot as plt
from uncertainties import unumpy

def read(topfile,powheg=True,mcfm=False):
	if powheg:
		return read_powheg(topfile,powheg)
	elif mcfm:
		return read_mcfm(topfile,mcfm)

def read_mcfm(topdir,mcfm=True,draw=False):
	# TODO output param
	if os.path.exists('mcfm.yoda'):
		os.remove('mcfm.yoda')
	for topfile in glob.glob(topdir + '/*.txt'):
		if not "TeV_" in topfile:
			continue
		df = pd.read_csv(topfile,sep=r'\s+',skiprows=5,header=None)
		df.columns = ['xlow','xhigh','sumw','sumw2sq']
		# 1000 to pb and divide by bin width
		df["sumw"] = df["sumw"]/1000
		df["sumw2sq"] = df["sumw2sq"]/1000
		df["sumw2"] = df["sumw2sq"]**2
		print(df)
		print(df["sumw"].sum())
		if draw:
			plot.data((df["xhigh"]+df['xlow'])/2,unumpy.uarray(df["sumw"],df["sumw2sq"])/(df["xhigh"]-df["xlow"]),logy=True)
			plot.show()

		type = topfile.replace('.txt','').split('TeV_')[-1].replace('_','')

		f = open("mcfm.yoda", "a")
		f.write("BEGIN YODA_HISTO1D_V2 /ALICE_2022_FSP/" + type + "\nPath: /ALICE_2022_FSP/" + type + "\nTitle: \nType: \"Histo1D\"\n---\n")
		f.close()
		df.astype(float).round(7).to_csv("mcfm.yoda",sep=' ',mode='a',header=False,index=False,columns=['xlow','xhigh','sumw','sumw2'])
		f = open("mcfm.yoda", "a")
		f.write("END YODA_HISTO1D_V2\n\n")
		f.close()
	return []



def read_powheg(topfile,powheg=True):
	topplots = []
	cur = TopPlot()
	cur_data = []
	lim = re.compile(r'set\s+limits\s+x\s+([0-9\.E\+-]+)\s+([0-9\.E\+-]+)\s+y\s+([0-9\.E\+-]+)\s+([0-9\.E\+-]+)')
	title = re.compile(r'title\s+(\w+)\s+"(.*)"')
	data = re.compile(r'([0-9\.Ee\+-]+)\s+([0-9\.Ee\+-]+)')
	join = re.compile(r'join')
	newplot = re.compile('newplot')
	with open(topfile) as topo_file:
		for line in topo_file:
			g = lim.search(line)
			if g is not None:
				cur.limits.xmin = float(g.group(1))
				cur.limits.xmax = float(g.group(2))
				cur.limits.ymin = float(g.group(3))
				cur.limits.ymax = float(g.group(4))
			g = title.search(line)
			if g is not None:
				cur.title.position = g.group(1)
				cur.title.text = g.group(2)
			g = data.search(line)
			if g is not None:
				cur_data.append([float( g.group(1)) ,float( g.group(2)) ])
			g = join.search(line)
			if g is not None:
				if len(cur_data) == 2:
					j = Join(cur_data[0][0],cur_data[0][1],cur_data[1][0],cur_data[1][1])
					cur.joins.append(j)
				else:
					cur.data = cur_data
				cur_data = []
			g = newplot.search(line)
			if g is not None:
				if powheg:
					cur.joins.append(Join(0,0,1,0))
					cur.joins.append(Join(0,0,0,1))
					cur._doskip(1)
				topplots.append(copy.deepcopy(cur))
				cur_data = []
				cur = TopPlot()
	return topplots





import numpy as np
from smpl import plot


class Title:
	def __init__(self,position="top",text=""):
		self.position = position
		self.text = text
class Limits:
	def __init__(self,xmin=0,xmax=1,ymin=0,ymax=1):
		self.xmin = xmin
		self.xmax = xmax
		self.ymin = ymin
		self.ymax = ymax
class Join:
	def __init__(self,x1,y1,x2,y2):
		self.x1=x1
		self.y1=y1
		self.x2=x2
		self.y2=y2
	def as_arrays(self):
		return [self.x1,self.x2],[self.y1,self.y2]

class TopPlot:

	def __init__(self,limits: Limits= Limits(), title:Title =Title(), data = None,joins = None,skipentries=0):
		if data is None:
			data = []
		if joins is None:
			joins = []
		self.limits = limits
		self.title = title
		if skipentries == 0:
			self.data = np.array(data)
		else:
			self.data = np.array(data)[skipentries:,:]
		self.joins = joins

	def _doskip(self,skips=1):
		self.data = np.array(self.data)[skips:,:]

	def xdata(self):
		return self.data[:,0]

	def ydata(self):
		return self.data[:,1]

	def grid(self,gridcolor="k",gridalpha=0.3,gridlinewidth=0.5,axes=None,**kwargs):
		if axes is None:
			axes = plot.gca()
		for join in self.joins:
			# print(join.x1,join.x2)
			axes.plot(*join.as_arrays(),color=gridcolor,alpha=gridalpha,linewidth=gridlinewidth)
	
	def auto(self,fmt="-",grid=False,**kwargs):
		plot.auto(self.xdata(),self.ydata() ,fmt=fmt,grid=grid,**kwargs)
		plot.title(self.title.text)
		self.grid(**kwargs)

	def fit(self,f,fmt="-",grid=False,**kwargs):
		plot.fit(self.xdata(),self.ydata() ,f,fmt=fmt,grid=grid,**kwargs)
		plot.title(self.title.text)
		self.grid(**kwargs)

	def plot(self,fmt="-",grid=False,**kwargs):
		plot.data(self.xdata(),self.ydata() ,fmt=fmt,grid=grid,**kwargs)
		plot.title(self.title.text)
		self.grid(**kwargs)

	def show(self,init=True,**kwargs):
		self.plot(init=init,**kwargs)
		plot.show()






#!/usr/bin/python3
import argparse
import math
import pytopdrawer
import matplotlib.pyplot as plt


def main():
	parser = argparse.ArgumentParser()
	parser.add_argument("topfile", type=str,help="url/file/string")
	parser.add_argument("-ns", "--noshow", action="store_true",help="do not show the plot")
	parser.add_argument("-ho", "--horizontal", action="store_true",help="horizontal layout")
	parser.add_argument("-v", "--vertical", action="store_true",help="vertical layout")
	parser.add_argument("-s", "--size", type=int,help="size of the plot", default=4)
	parser.add_argument("-o", "--output", help="output file", type=str, default=None)
	#parser.add_argument("-p", "--powheg", action="store_true",help="add powheg lines")
	parser.add_argument("-m","--mcfm", action="store_true",help="add mcfm lines")
	args = parser.parse_args()
	tops = pytopdrawer.read(args.topfile,not args.mcfm,args.mcfm)
	N = len(tops)
	cols = math.ceil(math.sqrt(N))   # Round up to ensure enough space
	rows = math.ceil(N / cols)       # Calculate rows based on columns
	if args.horizontal:
		rows = 1
		cols = N
	if args.vertical:
		rows = N
		cols = 1
	fig, axes = plt.subplots(rows,cols,  figsize=( cols*args.size,rows*args.size))
	ti = 0
	for i in range(rows):
		for j in range(cols):
			if rows == 1:
				a = axes[j]
			elif cols == 1:
				a = axes[i]
			else:
				a = axes[i][j]	
			tops[ti].plot(axes=a)
			ti += 1
			if ti == N:
				break

	if not args.noshow:
		plt.show()
	if args.output is not None:
		plt.savefig(args.output)
